"""Integration tests for hybrid structure functionality."""

# Skip hybrid integration tests for v0.9.2 release due to complexity
# Re-enabled hybrid integration tests for improved coverage
# pytestmark = pytest.mark.skip(reason="Hybrid integration tests require \
# complex setup - skipping for v0.9.2 release")
import shutil
import tempfile
from pathlib import Path
from unittest.mock import patch

import pytest

from src.claude_knowledge_catalyst.core.config import CKCConfig
from src.claude_knowledge_catalyst.core.hybrid_config import (
    NumberingSystem,
)
from src.claude_knowledge_catalyst.core.metadata import (
    MetadataManager,
)
from src.claude_knowledge_catalyst.core.structure_validator import validate_structure
from src.claude_knowledge_catalyst.sync.compatibility import (
    BackwardCompatibilityManager,
)
from src.claude_knowledge_catalyst.sync.hybrid_manager import HybridObsidianVaultManager


class TestHybridIntegration:
    """Test hybrid structure integration."""

    @pytest.fixture
    def temp_vault(self):
        """Create temporary vault directory."""
        temp_dir = tempfile.mkdtemp()
        vault_path = Path(temp_dir)
        yield vault_path
        shutil.rmtree(temp_dir)

    @pytest.fixture
    def hybrid_config(self, temp_vault):
        """Create hybrid configuration."""
        # Mock Path.cwd() to avoid "No such file or directory" errors during testing
        with patch("pathlib.Path.cwd", return_value=temp_vault):
            config = CKCConfig()
        config.project_root = temp_vault
        config.hybrid_structure.enabled = True
        config.hybrid_structure.numbering_system = NumberingSystem.TEN_STEP
        config.hybrid_structure.auto_classification = True
        config.hybrid_structure.structure_validation = True
        return config

    @pytest.fixture
    def legacy_config(self, temp_vault):
        """Create legacy configuration."""
        # Mock Path.cwd() to avoid "No such file or directory" errors during testing
        with patch("pathlib.Path.cwd", return_value=temp_vault):
            config = CKCConfig()
        config.project_root = temp_vault
        config.hybrid_structure.enabled = False
        config.hybrid_structure.numbering_system = NumberingSystem.SEQUENTIAL
        return config

    # Hybrid vault initialization test
    def test_hybrid_vault_initialization(self, temp_vault, hybrid_config):
        """Test hybrid vault initialization."""
        metadata_manager = MetadataManager()
        vault_manager = HybridObsidianVaultManager(
            temp_vault, metadata_manager, hybrid_config
        )

        # Initialize vault
        success = vault_manager.initialize_vault()
        assert success, "Vault initialization should succeed"

        # Check expected directories exist
        expected_dirs = [
            "_templates",
            "_attachments",
            "_scripts",
            "00_Catalyst_Lab",
            "10_Projects",
            "20_Knowledge_Base",
            "30_Wisdom_Archive",
            "Analytics",
            "Archive",
            "Evolution_Log",
        ]

        for dir_name in expected_dirs:
            dir_path = temp_vault / dir_name
            assert dir_path.exists(), f"Directory {dir_name} should exist"
            assert dir_path.is_dir(), f"{dir_name} should be a directory"

            # Check README exists
            readme_path = dir_path / "README.md"
            assert readme_path.exists(), f"README.md should exist in {dir_name}"

    def test_knowledge_base_subdirectories(self, temp_vault, hybrid_config):
        """Test Knowledge Base subdirectory creation."""
        metadata_manager = MetadataManager()
        vault_manager = HybridObsidianVaultManager(
            temp_vault, metadata_manager, hybrid_config
        )

        vault_manager.initialize_vault()

        # Check Knowledge Base subdirectories
        kb_subdirs = [
            "20_Knowledge_Base/Prompts/Templates",
            "20_Knowledge_Base/Prompts/Best_Practices",
            "20_Knowledge_Base/Code_Snippets/Python",
            "20_Knowledge_Base/Code_Snippets/JavaScript",
            "20_Knowledge_Base/Concepts/AI_Fundamentals",
            "20_Knowledge_Base/Resources/Documentation",
        ]

        for subdir in kb_subdirs:
            subdir_path = temp_vault / subdir
            assert subdir_path.exists(), f"Subdirectory {subdir} should exist"

    @pytest.mark.skip(reason="Temporary skip during hybrid system integration")
    def test_file_classification(self, temp_vault, hybrid_config):
        """Test automatic file classification."""
        metadata_manager = MetadataManager()
        vault_manager = HybridObsidianVaultManager(
            temp_vault, metadata_manager, hybrid_config
        )

        vault_manager.initialize_vault()

        # Create test files
        test_files = [
            ("prompt_test.md", {"tags": ["prompt"]}, "20_Knowledge_Base/Prompts"),
            (
                "python_code.md",
                {"tags": ["code", "python"]},
                "20_Knowledge_Base/Code_Snippets/Python",
            ),
            ("ai_concept.md", {"tags": ["concept"]}, "20_Knowledge_Base/Concepts"),
        ]

        for filename, metadata_dict, expected_dir in test_files:
            # Create test file
            test_file = temp_vault / filename
            test_content = f"""---
title: "{filename}"
tags: {metadata_dict["tags"]}
---

# {filename}

Test content for {filename}
"""
            test_file.write_text(test_content)

            # Sync file
            success = vault_manager.sync_file(test_file)
            assert success, f"Sync should succeed for {filename}"

            # Check file was placed correctly
            expected_path = temp_vault / expected_dir
            synced_files = list(expected_path.glob("*.md"))
            assert len(synced_files) > 0, f"File should be synced to {expected_dir}"

    def test_backward_compatibility(self, temp_vault, hybrid_config):
        """Test backward compatibility features."""
        # Create legacy structure first
        legacy_dirs = ["00_Inbox", "01_Projects", "02_Knowledge_Base"]
        for dir_name in legacy_dirs:
            (temp_vault / dir_name).mkdir()
            test_file = temp_vault / dir_name / "test.md"
            test_file.write_text("# Test\nTest content")

        # Note: legacy_support field removed in v2.0, compatibility is handled \
        # automatically

        compat_manager = BackwardCompatibilityManager(temp_vault, hybrid_config)

        # Ensure compatibility
        success = compat_manager.ensure_compatibility()
        assert success, "Compatibility setup should succeed"

        # Check that legacy directories are accessible
        for legacy_dir in legacy_dirs:
            legacy_path = temp_vault / legacy_dir
            assert legacy_path.exists(), (
                f"Legacy directory {legacy_dir} should be accessible"
            )

    def test_structure_validation(self, temp_vault, hybrid_config):
        """Test structure validation."""
        metadata_manager = MetadataManager()
        vault_manager = HybridObsidianVaultManager(
            temp_vault, metadata_manager, hybrid_config
        )

        vault_manager.initialize_vault()

        # Run validation
        validation_result = validate_structure(
            temp_vault, hybrid_config.hybrid_structure
        )

        assert validation_result.passed, "Structure validation should pass"
        assert len(validation_result.errors) == 0, "Should have no validation errors"

        # Check statistics
        stats = validation_result.statistics
        assert stats["total_directories"] > 0, "Should have directories"
        assert stats["numbering_system"] == "ten_step", "Should use ten-step numbering"

    def test_legacy_compatibility_manager(
        self, temp_vault, legacy_config, hybrid_config
    ):
        """Test legacy compatibility manager."""
        metadata_manager = MetadataManager()

        # Test with legacy config
        compat_manager = BackwardCompatibilityManager(temp_vault, legacy_config)
        vault_manager = compat_manager.get_appropriate_manager(metadata_manager)

        # Should get legacy manager
        assert not isinstance(vault_manager, HybridObsidianVaultManager)

        # Test with hybrid config
        compat_manager = BackwardCompatibilityManager(temp_vault, hybrid_config)
        vault_manager = compat_manager.get_appropriate_manager(metadata_manager)

        # Should get hybrid manager
        assert isinstance(vault_manager, HybridObsidianVaultManager)

    def test_config_migration(self):
        """Test configuration migration from v1.0 to v2.0."""
        # Create v1.0 config data
        v1_config_data = {
            "version": "1.0",
            "project_name": "test-project",
            "sync_targets": [],
            "auto_sync": True,
        }

        # Import should trigger automatic migration
        from src.claude_knowledge_catalyst.core.config_migration import (
            ConfigMigrationManager,
        )

        migration_manager = ConfigMigrationManager()
        v2_data = migration_manager._migrate_v1_to_v2(v1_config_data)

        # Check migration results
        assert v2_data["version"] == "2.0"
        assert v2_data["project_name"] == "test-project"
        assert "hybrid_structure" in v2_data
        assert "structure_migration_log" in v2_data

        # Check hybrid structure defaults
        hybrid_structure = v2_data["hybrid_structure"]
        assert not hybrid_structure["enabled"]  # Conservative default
        assert hybrid_structure["numbering_system"] == "sequential"
        assert hybrid_structure["legacy_support"]

    # End-to-end workflow integration test
    @pytest.mark.skip(reason="Temporary skip during hybrid system integration")
    def test_end_to_end_workflow(self, temp_vault, hybrid_config):
        """Test complete end-to-end workflow."""
        metadata_manager = MetadataManager()

        # 1. Initialize hybrid vault
        vault_manager = HybridObsidianVaultManager(
            temp_vault, metadata_manager, hybrid_config
        )
        success = vault_manager.initialize_vault()
        assert success

        # 2. Create test content files
        test_files = [
            ("experiment.md", "00_Catalyst_Lab", "experiment"),
            ("project_plan.md", "10_Projects", "project"),
            ("prompt_guide.md", "20_Knowledge_Base/Prompts/Templates", "prompt"),
            ("python_utils.md", "20_Knowledge_Base/Code_Snippets/Python", "code"),
            ("ai_concepts.md", "20_Knowledge_Base/Concepts", "concept"),
            ("resources.md", "20_Knowledge_Base/Resources", "resource"),
        ]

        for filename, expected_dir, category in test_files:
            # Create source file
            source_file = temp_vault / filename
            content = f"""---
title: "{filename}"
category: "{category}"
tags: ["{category}"]
status: "draft"
---

# {filename}

Content for {filename}
"""
            source_file.write_text(content)

            # Sync file
            success = vault_manager.sync_file(source_file)
            assert success

            # Verify placement
            expected_path = temp_vault / expected_dir
            synced_files = list(expected_path.glob("*.md"))
            assert len(synced_files) > 0, f"File should be in {expected_dir}"

        # 3. Validate final structure
        validation_result = validate_structure(
            temp_vault, hybrid_config.hybrid_structure
        )
        assert validation_result.passed

        # 4. Check statistics
        structure_info = vault_manager.get_structure_info()
        assert structure_info["hybrid_enabled"]
        assert structure_info["numbering_system"] == "ten_step"
        assert structure_info["auto_classification"]


if __name__ == "__main__":
    pytest.main([__file__])
